
import 'dart:async';
import 'dart:convert' as JSON;
import 'JsonExpression.dart';
import 'JsonToken.dart';
import 'JsonTokenizer.dart';
import 'Streamer.dart';

class Reader {
  JsonTokenizer tokenizer;
  Function _pipe;
  Function _trailing;
  Map<String, Function> _filters;
  Function _done;
  Function _fail;
  Reader(Streamer stream) {
    this._filters = new Map<String, Function>();
    this.tokenizer = new JsonTokenizer(stream);
    new Timer(new Duration(microseconds: 0), () {
      this._run();
    });
  }
  _run() {
    this.tokenizer.tokenize.listen((int response) {
      try {
        this._parse();
      } catch (e) {
        if (this._fail != null) {
          this._fail(e);
        }
      }
    }, onDone: () {
      if (this._done != null) {
        this._done();
      }
    }, onError: (error) {
      if (this._fail != null) {
        this._fail(error);
      }
    });
  }

  _parse() {
    List deletes = [];
    List keys = JsonToken.queue.keys.toList();
    var len = keys.length;
    Map<int, bool> trailing = new Map<int, bool>();
    first:
    for (var i = 0; i < len; i++) {
      var key = keys[i];
      var item = JsonToken.queue[key];
      if (item == null) {
        continue;
      }
      if (item.type == JsonToken.OBJECTSTART ||
          item.type == JsonToken.ARRAYSTART) {
        if ((i + 1) < len) {
          int _key = keys[i + 1];
          JsonToken _item = JsonToken.queue[_key];
          trailing[_item.parnterId] = true;
        }
      }
      if (item.type == JsonToken.OBJECTEND || item.type == JsonToken.ARRAYEND) {
        var _key = keys[i - 1];
        var _item = JsonToken.queue[_key];
        trailing[_item.parnterId] = false;
      }
      if (item.closed == true &&
          item.type == JsonToken.OBJECTSTART &&
          item.child == null) {
        List<JsonToken> stack = new List<JsonToken>();
        var start = item;
        stack.add(start);
        for (var j = i + 1; j < len; j++) {
          var key = keys[j];
          var item = JsonToken.queue[key];
          if (item.type == JsonToken.OBJECTEND &&
              item.parnterId == start.parnterId) {
            item.child = null;
            item.partner.child = null;
            stack.add(item);
            if (stack.length > 2) {
              //send data
              var obj = this._object(stack);

              if (this._pipe != null) {
                this._pipe(obj[0], obj[1], obj[2]);
              }
              for (var k in this._filters.keys) {
                if (new RegExp(k).hasMatch(stack[1].key)) {
                  this._filters[k](obj[0], obj[1], obj[2]);
                }
              }
              if (this._trailing != null) {
                var trailKeys = trailing.keys.toList();
                var k = trailKeys[trailKeys.length - 1];
                if (trailing[k]) {
                  if (trailing[k].runtimeType == bool) {
                    this._trailing(obj[0], obj[1], obj[2]);
                  }
                }
              }
            }
            i = j;
            continue first;
          } else {
            var next = i + 1;
            deletes.add(key);
            deletes.add(keys[next]);
            stack.add(item);
          }
        }
      } else if (item.type == JsonToken.VALUE && item.closed == true) {
        if (item.value == 'true') {
          item.value = true;
        } else if (item.value == 'false') {
          item.value = false;
        } else if (item.value == 'null') {
          item.value = null;
        }
        if (this._pipe != null) {
          //send data
          this._pipe(item.value.replaceAll(new RegExp(r'^"|"$'), ""), item.key,
              new List<JsonToken>().add(item));
        }
        for (var k in this._filters.keys) {
          if (new RegExp(k).hasMatch(item.key)) {
            this._filters[k](item.value.replaceAll(new RegExp(r'^"|"$'), ""),
                item.key, new List<JsonToken>().add(item));
          }
        }
        if (this._trailing != null) {
          for (var k in trailing.keys) {
            if (trailing[k]) {
              if (trailing[k].runtimeType == bool) {
                this._trailing(new RegExp(r'^"|"$'), item.key,
                    new List<JsonToken>().add(item));
              }
            }
          }
        }
        var next = i + 1;
        deletes.add(keys[next]);
        deletes.add(key);
      }
      if (trailing.keys.length > 0 &&
          (item.type == JsonToken.ARRAYSEPERATOR ||
              item.type == JsonToken.OBJECTEND) &&
          i > 2) {
        var _key_1 = keys[i - 3];
        var _key_2 = keys[i - 2];
        var _key_3 = keys[i - 1];
        var _item_1 = JsonToken.queue[_key_1];
        var _item_2 = JsonToken.queue[_key_2];
        var _item_3 = JsonToken.queue[_key_3];
        var _items = [_item_1, _item_2, _item_3];
        if (_item_1.type == JsonToken.KEY) {
          deletes.add(_key_1);
        }
        if (_item_3.type == JsonToken.VALUEPAIR) {
          deletes.add(_key_2);
          deletes.add(_key_3);
        }
        deletes.add(key);
        var obj = {};
        obj[_item_1.value] = _item_3.value.replaceAll(new RegExp(r'^"|"$'), "");
        if (_item_1.type == JsonToken.KEY &&
            _item_3.type == JsonToken.VALUEPAIR) {
          obj[_item_1.value] =
              _item_3.value.replaceAll(new RegExp(r'^"|"$'), "");
          if (this._trailing != null) {
            this._trailing(obj, _item_3.key, _items);
          }
          for (var k in this._filters.keys) {
            if (new RegExp(k).hasMatch(item.key)) {
              this._filters[k](obj, item.key, _items);
            }
          }
        }
      }
    }
    for (var m = 0; m < len; m++) {
      var key = keys[m];
      var item = JsonToken.queue[key];
      if (item == null) {
        continue;
      }
      if (item.parent != null) {
        if (item.parent.closed) {
          item.closed = true;
          deletes.add(key);
        } else if (item.closed == true) {
          deletes.add(key);
        }
      } else if (item.closed == true) {
        deletes.add(key);
      }
      if (item.type == JsonToken.OBJECTSEPERATOR && !item.closed) {
        if ((m - 1) < len) {
          var previous = keys[m - 1];
          var sub = JsonToken.queue[previous];
          if (sub.type != JsonToken.KEY) {
            deletes.add(key);
          }
        }
      }
      if (item.type == JsonToken.KEY) {
        int next = m + 1;
        int nextnext = next + 1;
        if (nextnext < len) {
          var sub = JsonToken.queue[keys[nextnext]];
          if (sub.closed == true) {
            deletes.add(key);
          }
        }
      }
    }
    deletes.forEach((v) {
      if (JsonToken.queue.containsKey(v)) {
        JsonToken.queue.remove(v);
      }
    });
  }

  Reader pipe(Function func(dynamic value, String key, List<JsonToken> item)) {
    var f = (dynamic value, String key, List<JsonToken> item) {
      func(value, key, item);
    };
    this._pipe = f;
    return this;
  }

  Reader trailing(
      Function func(dynamic value, String key, List<JsonToken> item)) {
    var f = (dynamic value, String key, List<JsonToken> item) {
      func(value, key, item);
    };
    this._trailing = f;
    return this;
  }

  Reader done(Function func) {
    this._done = func;
    return this;
  }

  Reader fail(Function func(dynamic err)) {
    this._fail = func;
    return this;
  }

  Reader filter(String expression,
      Function func(dynamic value, String key, List<JsonToken> item)) {
    this._filters[new JsonExpression(expression).toPath()] = func;
    return this;
  }

  List<dynamic> _object(List<JsonToken> arr) {
    var alen = arr.length;
    var string = '' + arr[0].value;
    if (alen > 2) {
      var key = arr[1].key;
      for (var k = 1; k < alen; k++) {
        String v = arr[k].value;
        string += v;
      }
      return [JSON.jsonDecode(string), key, arr];
    } else {
      return [{}, null, arr];
    }
  }
}
